@@ -2,26 +2,36 @@
 # Timeseries visualizer for Electrocorticography (ECoG) signals stored in NWB files.
 # Developed by:
 # Luiz Tauffer, Ben Dichter
-# https://github.com/bendichter/ecogVIS
+# https://github.com/catalystneuro/ecogVIS
 #
 import sys
 import os
 import numpy as np
 import datetime
 import glob
+import yaml
+from pathlib import Path
 
-from PyQt5 import QtCore, QtGui, Qt
+from PyQt5 import QtCore, QtGui
 from PyQt5.QtCore import QCoreApplication
 from PyQt5.QtWidgets import (QWidget, QLabel, QLineEdit, QMessageBox, QHBoxLayout,
-    QApplication, QPushButton, QVBoxLayout, QGroupBox, QGridLayout, QComboBox,
-    QInputDialog, QFileDialog, QMainWindow, QAction, QStackedLayout)
+                             QApplication, QPushButton, QVBoxLayout, QGroupBox,
+                             QGridLayout, QComboBox, QInputDialog, QFileDialog,
+                             QMainWindow, QAction, QStackedLayout)
 import pyqtgraph as pg
 
 from ecogvis.functions.subFunctions import TimeSeriesPlotter
-from ecogvis.functions.subDialogs import (CustomIntervalDialog, SelectChannelsDialog,
-    SpectralChoiceDialog, NoHighGammaDialog, NoPreprocessedDialog, NoTrialsDialog,
-    ExitDialog, ERPDialog, HighGammaDialog, PeriodogramGridDialog, AudioEventDetection,
-    PreprocessingDialog, NoRawDialog, NoSpectrumDialog, NoAudioDialog, ExistIntervalsDialog)
+from ecogvis.functions.misc_dialogs import (CustomIntervalDialog, SelectChannelsDialog,
+                                            SpectralChoiceDialog, NoHighGammaDialog,
+                                            NoPreprocessedDialog, NoTrialsDialog,
+                                            ExitDialog, HighGammaDialog, LoadHTKDialog,
+                                            PeriodogramGridDialog, NoSpectrumDialog,
+                                            PreprocessingDialog, NoRawDialog,
+                                            NoAudioDialog, ExistIntervalsDialog)
+from ecogvis.functions.audio_event_detection import AudioEventDetection
+from ecogvis.functions.event_related_potential import ERPDialog
+from ecogvis.functions.save_to_nwb import SaveToNWBDialog
+from ecogvis.functions.nwb_copy_file import nwb_copy_file
 
 annotationAdd_ = False
 annotationDel_ = False
@@ -36,9 +46,17 @@ intervalsDict_ = {'invalid': {'type': 'invalid',
 
 
 class Application(QMainWindow):
+    """
+    Main EcogVIS window.
+
+    Parameters
+    ----------
+    source_path : str or path
+        Path to the source of data, nwb file or htk directory.
+    """
     keyPressed = QtCore.pyqtSignal(QtCore.QEvent)
 
-    def __init__(self, filename, parent=None):
+    def __init__(self, source_path=None, metafile=None, session=None):
         super().__init__()
         # Enable anti-aliasing for prettier plots
         pg.setConfigOptions(antialias=True)
@@ -48,10 +66,16 @@ class Application(QMainWindow):
         self.resize(1000, 600)
         self.setWindowTitle('ecogVIS')
 
-        # e.g.: /home/User/freesurfer_subjects/Subject_XX.nwb
-        if not os.path.isfile(filename):
-            filename = self.open_file()
-        self.file = filename
+        # Check source of data
+        if not os.path.exists(source_path):
+            source_path = self.open_file()
+        self.source_path = Path(source_path).absolute()
+
+        # Check metadata file
+        self.metadata = None
+        if metafile is not None:
+            with open(metafile) as f:
+                self.metadata = yaml.safe_load(f)
 
         self.error = None
         self.keyPressed.connect(self.on_key)
@@ -61,12 +85,16 @@ class Application(QMainWindow):
         self.show()
 
         # opens dialog for session name
-        text = 'Enter session name:\n(e.g. your_name)'
-        uinp, ok = QInputDialog.getText(None, 'Choose session', text)
-        if ok:
-            self.current_session = uinp
+        if session is None:
+            text = 'Enter session name:\n(e.g. your_name)'
+            self.enter_sesssion_name_dialog = QInputDialog()
+            uinp, ok = self.enter_sesssion_name_dialog.getText(None, 'Choose session', text)
+            if ok:
+                self.current_session = uinp
+            else:
+                self.current_session = 'default'
         else:
-            self.current_session = 'default'
+            self.current_session = session
 
         # Run the main function
         self.model = TimeSeriesPlotter(self)
@@ -75,10 +103,12 @@ class Application(QMainWindow):
         """Before exiting, checks if there are any unsaved changes and inform the user."""
         w = ExitDialog(self)
         if w.value == -1:  # just exit
+            self.model.close_nwbfile()
             event.accept()
         elif w.value == 1:  # save and exit
             self.AnnotationSave()
             self.IntervalSave()
+            self.model.close_nwbfile()
             event.accept()
         elif w.value == 0:  # ignore
             event.ignore()
@@ -108,9 +138,9 @@ class Application(QMainWindow):
         elif event.key() == QtCore.Qt.Key_PageDown:
             self.model.channel_Scroll_Down('page')
         elif event.key() == QtCore.Qt.Key_Left:
-            self.model.time_scroll(scroll=-1/3)
+            self.model.time_scroll(scroll=-1 / 3)
         elif event.key() == QtCore.Qt.Key_Right:
-            self.model.time_scroll(scroll=1/3)
+            self.model.time_scroll(scroll=1 / 3)
         event.accept()
 
     def init_gui(self):
@@ -122,12 +152,17 @@ class Application(QMainWindow):
         change_session_file = QAction('Change Session', self)
         fileMenu.addAction(change_session_file)
         change_session_file.triggered.connect(self.change_session)
-        action_open_file = QAction('Open Another File', self)
-        fileMenu.addAction(action_open_file)
-        action_open_file.triggered.connect(lambda: self.open_another_file(None))
-        #action_save_file = QAction('Save to NWB', self)
-        #fileMenu.addAction(action_save_file)
-        #action_save_file.triggered.connect(self.save_file)
+        open_tools_menu = fileMenu.addMenu('Open from')
+        action_open_nwb = QAction('NWB', self)
+        open_tools_menu.addAction(action_open_nwb)
+        action_open_nwb.triggered.connect(lambda: self.open_another_file(None))
+        action_open_htk = QAction('HTK dir', self)
+        open_tools_menu.addAction(action_open_htk)
+        action_open_htk.triggered.connect(self.open_htk_dir)
+
+        action_save_new_file = QAction('Save to NWB', self)
+        fileMenu.addAction(action_save_new_file)
+        action_save_new_file.triggered.connect(self.save_file)
 
         toolsMenu = mainMenu.addMenu('Tools')
         action_load_annotations = QAction('Load Annotations', self)
@@ -162,12 +197,14 @@ class Application(QMainWindow):
         action_about.triggered.connect(self.about)
 
         # Buttons and controls vertical box layout -----------------------------
-        #Block change buttons
+        # Block change buttons
         self.qlabelBlocks = QLabel('Move Block')
         self.pushBlock_0 = QPushButton('<')
         self.pushBlock_0.clicked.connect(lambda: self.change_block(-1))
         self.pushBlock_1 = QPushButton('>')
         self.pushBlock_1.clicked.connect(lambda: self.change_block(1))
+        self.pushBlock_0.setEnabled(False)
+        self.pushBlock_1.setEnabled(False)
         grid0 = QGridLayout()
         grid0.addWidget(self.pushBlock_0, 0, 0, 1, 1)
         grid0.addWidget(self.qlabelBlocks, 0, 1, 1, 3)
@@ -208,7 +245,7 @@ class Application(QMainWindow):
         self.push2_3 = QPushButton('Save')
         self.push2_3.clicked.connect(self.IntervalSave)
 
-        #Bad channels buttons
+        # Bad channels buttons
         qlabelBadChannels = QLabel('Bad channels:')
         self.push3_1 = QPushButton('Add')
         self.push3_1.clicked.connect(self.add_badchannel)
@@ -220,7 +257,7 @@ class Application(QMainWindow):
         # Get channel by brain region
         self.push4_0 = QPushButton('Select regions')
         self.push4_0.clicked.connect(self.ChannelSelect)
-        #Periodogram
+        # Periodogram
         self.push5_0 = QPushButton('Periodogram')
         self.push5_0.clicked.connect(self.PeriodogramSelect)
         # One-click preprocess signals
@@ -254,7 +291,6 @@ class Application(QMainWindow):
         grid1.addWidget(self.push7_0, 7, 3, 1, 3)
         panel1.setLayout(grid1)
 
-
         # Traces panel ---------------------------------------------------------
         panel2 = QGroupBox('Traces')
         panel2.setFixedWidth(250)
@@ -276,7 +312,6 @@ class Application(QMainWindow):
         grid2.setAlignment(QtCore.Qt.AlignTop)
         panel2.setLayout(grid2)
 
-
         # Plot controls panel --------------------------------------------------
         qlabel1 = QLabel('Top')
         qlabel2 = QLabel('Bottom')
@@ -357,24 +392,23 @@ class Application(QMainWindow):
         self.vbox1.addWidget(panel3)
         self.vbox1.addStretch()
 
-
         # Time series plots Vertical Box ---------------------------------------
         vb = CustomViewBox(self)
-        self.win1 = pg.PlotWidget(viewBox = vb)   #middle signals plot
-        self.win2 = pg.PlotWidget()               #upper horizontal bar
-        self.win3 = pg.PlotWidget()               #lower audio plot
+        self.win1 = pg.PlotWidget(viewBox=vb)   # middle signals plot
+        self.win2 = pg.PlotWidget()               # upper horizontal bar
+        self.win3 = pg.PlotWidget()               # lower audio plot
         self.win1.setBackground(self.palette().color(QtGui.QPalette.Background))
         self.win2.setBackground(self.palette().color(QtGui.QPalette.Background))
         self.win3.setBackground(self.palette().color(QtGui.QPalette.Background))
-        self.win1.setMouseEnabled(x = False, y = False)
-        self.win2.setMouseEnabled(x = False, y = False)
-        self.win3.setMouseEnabled(x = False, y = False)
+        self.win1.setMouseEnabled(x=False, y=False)
+        self.win2.setMouseEnabled(x=False, y=False)
+        self.win3.setMouseEnabled(x=False, y=False)
         self.win1.hideButtons()
         self.win2.hideButtons()
         self.win3.hideButtons()
-        #self.win2.hideAxis('left')
+        # self.win2.hideAxis('left')
         self.win2.hideAxis('bottom')
-        #self.win3.hideAxis('left')
+        # self.win3.hideAxis('left')
         self.win3.hideAxis('bottom')
 
         form_3 = QGridLayout()
@@ -395,39 +429,39 @@ class Application(QMainWindow):
 
         # Creates a horizontal box layout and populate it with panels ----------
         self.hbox = QHBoxLayout(self.centralwidget)
-        self.hbox.addLayout(self.vbox1)    #add panels first
-        self.hbox.addLayout(self.vbox2)    #add plots second
-
+        self.hbox.addLayout(self.vbox1)    # add panels first
+        self.hbox.addLayout(self.vbox2)    # add plots second
 
     def change_block(self, move):
         """Move between Block files for the same subject."""
-        fpath = os.path.split(os.path.abspath(self.file))[0] #file directory
-        fname = os.path.split(os.path.abspath(self.file))[1] #current file
+        fpath = os.path.split(os.path.abspath(self.source_path))[0]  # current directory
+        fname = os.path.split(os.path.abspath(self.source_path))[1]  # current file
         pre, _ = fname.split('_B')
-        flist = glob.glob(fpath+'/'+pre+'_B*.nwb')
-        curr_ind = flist.index(self.file)
-        if curr_ind+move == len(flist):
+        flist = glob.glob(fpath + '/' + pre + '_B*.nwb')
+        curr_ind = flist.index(self.source_path)
+        if curr_ind + move == len(flist):
             new_file = flist[0]
         else:
-            new_file = flist[curr_ind+move]
+            new_file = flist[curr_ind + move]
         self.open_another_file(filename=new_file)
 
-
     def open_file(self):
         """Opens initial file."""
         filename, _ = QFileDialog.getOpenFileName(None, 'Open file', '', "(*.nwb)")
         return filename
 
-
     def open_another_file(self, filename=None):
         """Opens another NWB file."""
-        if filename is None: # Opens new file dialog
+        if filename is None:  # Opens new file dialog
             filename, _ = QFileDialog.getOpenFileName(None, 'Open file', '', "(*.nwb)")
         if os.path.isfile(filename):
-            self.file = filename
+            self.source_path = Path(filename)
             # Reset file specific variables on GUI
             self.combo3.setCurrentIndex(self.combo3.findText('raw'))
             self.combo4.clear()
+            self.push5_0.setEnabled(True)
+            self.push6_0.setEnabled(True)
+            self.push7_0.setEnabled(True)
             self.qline0.setText('16')
             self.qline1.setText('1')
             self.qline2.setText('0.01')
@@ -439,138 +473,165 @@ class Application(QMainWindow):
             # Rebuild the model
             self.model = TimeSeriesPlotter(self)
 
+    def open_htk_dir(self):
+        """
+        Dialog for user selection of directory containing block of HTK files.
+        Assembles a nwbfile object (in memory) from htk files.
+        """
+        w = LoadHTKDialog(parent=self)
+        if w.value == 1 and w.htk_config:
+            self.source_path = Path(w.htk_config['ecephys_path'])
+            self.metadata = w.htk_config['metadata']
+            # Reset file specific variables on GUI
+            self.combo3.setCurrentIndex(self.combo3.findText('raw'))
+            self.combo4.clear()
+            self.push5_0.setEnabled(True)
+            self.push6_0.setEnabled(True)
+            self.push7_0.setEnabled(True)
+            self.qline0.setText('16')
+            self.qline1.setText('1')
+            self.qline2.setText('0.01')
+            self.qline3.setText('2')
+            self.qline4.setText('1')
+            self.win1.clear()
+            self.win2.clear()
+            self.win3.clear()
+            # Rebuild the model
+            self.model = TimeSeriesPlotter(par=self, htk_config=w.htk_config)
 
     def save_file(self):
-        """Saves latest alterations to NWB file. TO BE IMPLEMENTED."""
-        print('Save file to NWB - to be implemented')
-
+        """
+        Saves chosen data and info to new NWB file.
+        The dialog assembles a dictionary with fields to be copied from the current
+        nwb file to the new file. This dictionary is passed to nwb_copy_file()
+        function.
+        """
+        save_dialog = SaveToNWBDialog(parent=self)
+        if save_dialog.value:
+            nwb_copy_file(
+                old_file=self.source_path,
+                new_file=save_dialog.newfile,
+                cp_objs=save_dialog.cp_objs,
+                save_to_file=True
+            )
 
     def change_session(self):
         """Changes session name."""
-        text = 'Current session is: '+self.current_session+'\nEnter new session name:'
+        text = 'Current session is: ' + self.current_session + '\nEnter new session name:'
         uinp, ok = QInputDialog.getText(None, 'Change session', text)
         if ok:
             self.current_session = uinp
-            fname = os.path.split(os.path.abspath(self.file))[1] #file
-            self.setWindowTitle('ecogVIS - '+fname+' - '+self.current_session)
-
+            fname = os.path.split(os.path.abspath(self.source_path))[1]  # file
+            self.setWindowTitle('ecogVIS - ' + fname + ' - ' + self.current_session)
 
     def load_annotations(self):
         """Loads annotations from file, calls function to paint them."""
         fname, aux = QFileDialog.getOpenFileName(self, 'Open file', '', "(*.csv)")
-        if fname!='':
+        if fname != '':
             self.model.AnnotationLoad(fname=fname)
 
-
     def load_intervals(self):
         """Loads intervals from file, calls function to paint them."""
         fname, aux = QFileDialog.getOpenFileName(self, 'Open file', '', "(*.csv)")
-        if fname!='':
+        if fname != '':
             self.model.IntervalLoad(fname=fname)
 
-
     def add_badchannel(self):
         """Opens dialog for user input of channels to mark as bad."""
         text = 'Channel number: \n(e.g.: 3, 5, 8-12)'
         uinp, ok = QInputDialog.getText(None, 'Add as bad channel', text)
         if ok:
-            uinp = uinp.replace(' ','')  #removes blank spaces
-            ch_str = uinp.split(',')     #splits csv
+            uinp = uinp.replace(' ', '')  # removes blank spaces
+            ch_str = uinp.split(',')     # splits csv
             try:
                 ch_list = []
                 for elem in ch_str:
-                    if '-' in elem:   #if given a range e.g. 7-12
+                    if '-' in elem:   # if given a range e.g. 7-12
                         elem_lims = elem.split('-')
-                        seq = range(int(elem_lims[0])-1,int(elem_lims[1]))
+                        seq = range(int(elem_lims[0]) - 1, int(elem_lims[1]))
                         ch_list.extend(seq)
-                    else:             #if given a single value
-                        ch_list.append(int(elem)-1)
+                    else:             # if given a single value
+                        ch_list.append(int(elem) - 1)
                 self.model.BadChannelAdd(ch_list=ch_list)
             except Exception as ex:
                 print(str(ex))
 
-
     def del_badchannel(self):
         """Opens dialog for user input of channels to unmark as bad."""
         text = 'Channel number: \n(e.g.: 3, 5, 8-12)'
         uinp, ok = QInputDialog.getText(None, 'Delete bad channel', text)
         if ok:
-            uinp = uinp.replace(' ','')  #removes blank spaces
-            ch_str = uinp.split(',')     #splits csv
+            uinp = uinp.replace(' ', '')  # removes blank spaces
+            ch_str = uinp.split(',')     # splits csv
             try:
                 ch_list = []
                 for elem in ch_str:
-                    if '-' in elem:   #if given a range e.g. 7-12
+                    if '-' in elem:   # if given a range e.g. 7-12
                         elem_lims = elem.split('-')
-                        seq = range(int(elem_lims[0])-1,int(elem_lims[1]))
+                        seq = range(int(elem_lims[0]) - 1, int(elem_lims[1]))
                         ch_list.extend(seq)
-                    else:             #if given a single value
-                        ch_list.append(int(elem)-1)
+                    else:             # if given a single value
+                        ch_list.append(int(elem) - 1)
                 self.model.BadChannelDel(ch_list=ch_list)
             except Exception as ex:
                 print(str(ex))
 
-
     def save_badchannel(self):
         """Saves bad channels to file."""
         self.model.BadChannelSave()
 
-
     def spectral_decomposition(self):
         """Opens Spectral decomposition dialog."""
         w = SpectralChoiceDialog(self)
-        if w.value==1:       # If new data was created
+        if w.value == 1:       # If new data was created
             self.model.refresh_file()
 
-
     def event_related_potential(self):
         """Opens ERP window."""
         open_dialog = True
-        try: #Checks if file contains proper intervals information
+        try:  # Checks if file contains proper intervals information
             speaker_start_times = self.model.nwb.intervals['TimeIntervals_speaker']['start_time'].data
             speaker_stop_times = self.model.nwb.intervals['TimeIntervals_speaker']['stop_time'].data
             mic_start_times = self.model.nwb.intervals['TimeIntervals_mic']['start_time'].data
             mic_stop_times = self.model.nwb.intervals['TimeIntervals_mic']['stop_time'].data
-        except: #if it fails, raises a no intervals data message
+        except:  # if it fails, raises a no intervals data message
             NoTrialsDialog()
             open_dialog = False
-        try:  #Tries to load High Gamma and intervals data
+        try:  # Tries to load High Gamma and intervals data
             hg = self.model.nwb.processing['ecephys'].data_interfaces['high_gamma']
-        except: #if it fails, raises a no high gamma message
+        except:  # if it fails, raises a no high gamma message
             NoHighGammaDialog()
             open_dialog = False
-        #If all tries passed, open ERP
+        # If all tries passed, open ERP
         if open_dialog:
             w = ERPDialog(parent=self)
 
-
     def audio_event_detection(self):
         """Opens Audio Event Detection window."""
-         #Test if trials already exist
+        # Test if trials already exist
         if 'TimeIntervals_speaker' not in self.model.nwb.intervals:
-            #Test if file contains audio signals
-            if ('microphone' in self.model.nwb.acquisition) and ('speaker1' in self.model.nwb.stimulus):
+            # Test if file contains audio signals
+            # if ('microphone' in self.model.nwb.acquisition) and ('speaker1' in self.model.nwb.stimulus):
+            if any(name in self.model.nwb.acquisition for name in ['anin4', 'microphone']) and ('speaker1' in self.model.nwb.stimulus):
                 w = AudioEventDetection(parent=self)
-                if w.value == 1:  #Trial times were detected
+                if w.value == 1:  # Trial times were detected
                     self.model.refresh_file()  # re-opens the file, now with new data
             else:
                 NoAudioDialog()
         else:
             ExistIntervalsDialog()
 
-
     def about(self):
         """About dialog."""
         msg = QMessageBox()
         msg.setWindowTitle("About ecogVIS")
         msg.setIcon(QMessageBox.Information)
-        msg.setText("Version: 1.0.0 \n"+
+        msg.setText("Version: 1.0.0 \n"
                     "Timeseries visualizer for ECoG signals stored in NWB files.\n ")
         msg.setInformativeText("<a href='https://github.com/luiztauffer/ecogVIS/'>ecogVIS Github page</a>")
         msg.setStandardButtons(QMessageBox.Ok)
         msg.exec_()
 
-
     def reset_buttons(self):
         """Controls appearance and active state of panel buttons."""
         global intervalAdd_
@@ -578,7 +639,7 @@ class Application(QMainWindow):
         global annotationAdd_
         global annotationDel_
 
-        #self.active_mode = 'default', 'intervalAdd', 'intervalDel'
+        # self.active_mode = 'default', 'intervalAdd', 'intervalDel'
         if self.active_mode != 'intervalAdd':
             self.push2_1.setChecked(False)
             intervalAdd_ = False
@@ -595,8 +656,7 @@ class Application(QMainWindow):
             self.push1_2.setChecked(False)
             annotationDel_ = False
 
-
-    ## Annotation functions ----------------------------------------------------
+    # Annotation functions ----------------------------------------------------
     def AnnotationColor(self):
         """Selects annotation color."""
         global annotationColor_
@@ -605,7 +665,7 @@ class Application(QMainWindow):
     def AnnotationAdd(self):
         """Add new annotation at point selected with mouse right-click."""
         global annotationAdd_
-        if self.push1_1.isChecked():  #if button is pressed down
+        if self.push1_1.isChecked():  # if button is pressed down
             self.active_mode = 'annotationAdd'
             self.reset_buttons()
             annotationAdd_ = True
@@ -616,7 +676,7 @@ class Application(QMainWindow):
     def AnnotationDel(self):
         """Deletes annotations chosen with mouse right-click."""
         global annotationDel_
-        if self.push1_2.isChecked():  #if button is pressed down
+        if self.push1_2.isChecked():  # if button is pressed down
             self.active_mode = 'annotationDel'
             self.reset_buttons()
             annotationDel_ = True
@@ -633,8 +693,7 @@ class Application(QMainWindow):
         except Exception as ex:
             self.log_error(str(ex))
 
-
-    ## Interval functions ------------------------------------------------------
+    # Interval functions ------------------------------------------------------
     def IntervalType(self):
         """Add new interval type."""
         global intervalType_
@@ -644,26 +703,25 @@ class Application(QMainWindow):
         if item == 'add custom':
             w = CustomIntervalDialog()
             int_type, color = w.getResults()
-            if len(int_type)>0:    # If user chose a valid name for the new interval type
+            if len(int_type) > 0:    # If user chose a valid name for the new interval type
                 curr_ind = self.combo2.currentIndex()
                 self.combo2.setItemText(curr_ind, int_type)
                 self.combo2.addItem('add custom')
                 intervalType_ = int_type
-                intervalsDict_[intervalType_] = {'type':intervalType_,
-                                                 'session':self.current_session,
-                                                 'color':color,
-                                                 'counts':0}
+                intervalsDict_[intervalType_] = {'type': intervalType_,
+                                                 'session': self.current_session,
+                                                 'color': color,
+                                                 'counts': 0}
                 self.combo2.setCurrentIndex(curr_ind)
             else:
                 self.combo2.setCurrentIndex(0)
         else:
             intervalType_ = item
 
-
     def IntervalAdd(self):
         """Adds intervals with mouse click-and-drag at time series plot."""
         global intervalAdd_
-        if self.push2_1.isChecked():  #if button is pressed down
+        if self.push2_1.isChecked():  # if button is pressed down
             self.active_mode = 'intervalAdd'
             self.reset_buttons()
             intervalAdd_ = True
@@ -671,11 +729,10 @@ class Application(QMainWindow):
             self.active_mode = 'default'
             self.reset_buttons()
 
-
     def IntervalDel(self):
         """Deletes intervals chosen with mouse right-click."""
         global intervalDel_
-        if self.push2_2.isChecked():  #if button is pressed down
+        if self.push2_2.isChecked():  # if button is pressed down
             self.active_mode = 'intervalDel'
             self.reset_buttons()
             intervalDel_ = True
@@ -683,7 +740,6 @@ class Application(QMainWindow):
             self.active_mode = 'default'
             self.reset_buttons()
 
-
     def IntervalSave(self):
         """Saves current set of intervals to file."""
         self.active_mode = 'default'
@@ -693,7 +749,6 @@ class Application(QMainWindow):
         except Exception as ex:
             self.log_error(str(ex))
 
-
     def ChannelSelect(self):
         """Opens dialog to select channels from specific brain regions."""
         self.active_mode = 'default'
@@ -703,8 +758,8 @@ class Application(QMainWindow):
         all_locs = self.model.nwb.electrodes['location'][:]
         self.model.channels_mask = np.zeros(len(all_locs))
         for loc in w.choices:
-            self.model.channels_mask += all_locs==loc
-        #Indices of channels from chosen regions
+            self.model.channels_mask += all_locs == np.array(loc)
+        # Indices of channels from chosen regions
         self.model.channels_mask_ind = np.where(self.model.channels_mask)[0]
         self.model.nChTotal = len(self.model.channels_mask_ind)
         # Reset channels span control
@@ -714,14 +769,13 @@ class Application(QMainWindow):
         self.qline0.setText(str(self.model.lastCh))
         self.qline1.setText(str(self.model.firstCh))
         # Update signals plot
-        self.model.selectedChannels = self.model.channels_mask_ind[self.model.firstCh-1:self.model.lastCh]
+        self.model.selectedChannels = self.model.channels_mask_ind[self.model.firstCh - 1:self.model.lastCh]
         self.model.refreshScreen()
 
-
     def PeriodogramSelect(self):
         """Opens Periodogram grid window. Checks if PSD data is present on file
         and, if not, asks the user if it should be calculated."""
-        if self.combo3.currentText()=='raw':
+        if self.combo3.currentText() == 'raw':
             try:  # tries to read fft and welch spectral data
                 psd_fft = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_fft_raw']
                 psd_welch = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_welch_raw']
@@ -734,25 +788,24 @@ class Application(QMainWindow):
                     psd_fft = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_fft_raw']
                     psd_welch = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_welch_raw']
                     PeriodogramGridDialog(self)
-        elif self.combo3.currentText()=='preprocessed':
-            try:  #tries to read fft and welch spectral data
+        elif self.combo3.currentText() == 'preprocessed':
+            try:  # tries to read fft and welch spectral data
                 psd_fft = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_fft_preprocessed']
                 psd_welch = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_welch_preprocessed']
                 PeriodogramGridDialog(self)
             except:
                 w = NoSpectrumDialog(self, 'preprocessed')
-                if w.val == 1:  #PSD was calculated
+                if w.val == 1:  # PSD was calculated
                     self.model.refresh_file()  # re-opens the file, now with new data
                     self.combo3.setCurrentIndex(self.combo3.findText('preprocessed'))
                     psd_fft = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_fft_preprocessed']
                     psd_welch = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_welch_preprocessed']
                     PeriodogramGridDialog(self)
 
-
     def Preprocess(self):
         """Opens Preprocessing dialog."""
         w = PreprocessingDialog(self)
-        if w.value==1:       # If new data was created
+        if w.value == 1:       # If new data was created
             self.model.refresh_file()        # re-opens the file, now with new data
             self.combo3.setCurrentIndex(self.combo3.findText('preprocessed'))
             self.voltage_time_series()
@@ -760,7 +813,7 @@ class Application(QMainWindow):
     def CalcHighGamma(self):
         """Opens calculate High Gamma dialog."""
         w = HighGammaDialog(self)
-        if w.value==1:       # If new data was created
+        if w.value == 1:       # If new data was created
             self.open_another_file(filename=w.new_fname)
 
     def voltage_time_series(self):
@@ -770,7 +823,7 @@ class Application(QMainWindow):
         - 'preprocessed': Preprocessed voltage traces, stored in nwb.processing['ecephys'].data_interfaces['LFP'].electrical_series['preprocessed']
         - 'high gamma': High Gamma estimation traces, stored in nwb.processing['ecephys'].data_interfaces['high_gamma']
         """
-        if self.combo3.currentText()=='raw':
+        if self.combo3.currentText() == 'raw':
             lis = list(self.model.nwb.acquisition.keys())
             there_is_raw = False
             for i in lis:  # Check if there is ElectricalSeries in acquisition group
@@ -780,9 +833,9 @@ class Application(QMainWindow):
             if there_is_raw:
                 self.model.plot_panel = 'voltage_raw'
                 self.model.plotData = self.model.source.data
-                self.model.nBins = self.model.plotData.shape[0]     #total number of bins
-                self.model.fs_signal = self.model.source.rate     #sampling frequency [Hz]
-                self.model.tbin_signal = 1/self.model.fs_signal #time bin duration [seconds]
+                self.model.nBins = self.model.plotData.shape[0]     # total number of bins
+                self.model.fs_signal = self.model.source.rate     # sampling frequency [Hz]
+                self.model.tbin_signal = 1 / self.model.fs_signal  # time bin duration [seconds]
                 self.push5_0.setEnabled(True)
                 self.push6_0.setEnabled(True)
                 self.push7_0.setEnabled(False)
@@ -790,14 +843,14 @@ class Application(QMainWindow):
                 self.combo3.setCurrentIndex(self.combo3.findText('high gamma'))
                 self.voltage_time_series()
                 NoRawDialog()
-        elif self.combo3.currentText()=='preprocessed':
-            try:   #if preprocessed signals already exist on NWB file
+        elif self.combo3.currentText() == 'preprocessed':
+            try:   # if preprocessed signals already exist on NWB file
                 self.model.source = self.model.nwb.processing['ecephys'].data_interfaces['LFP'].electrical_series['preprocessed']
                 self.model.plotData = self.model.source.data
                 self.model.plot_panel = 'voltage_preprocessed'
                 self.model.nBins = self.model.source.data.shape[0]
                 self.model.fs_signal = self.model.source.rate
-                self.model.tbin_signal = 1/self.model.fs_signal
+                self.model.tbin_signal = 1 / self.model.fs_signal
                 self.push5_0.setEnabled(True)
                 self.push6_0.setEnabled(False)
                 self.push7_0.setEnabled(True)
@@ -805,33 +858,31 @@ class Application(QMainWindow):
                 self.combo3.setCurrentIndex(self.combo3.findText('raw'))
                 self.voltage_time_series()
                 NoPreprocessedDialog()
-        elif self.combo3.currentText()=='high gamma':
-            try:     #if high gamma already exists on NWB file
+        elif self.combo3.currentText() == 'high gamma':
+            try:     # if high gamma already exists on NWB file
                 self.model.source = self.model.nwb.processing['ecephys'].data_interfaces['high_gamma']
                 self.model.plotData = self.model.source.data
                 self.model.plot_panel = 'spectral_power'
                 self.model.nBins = self.model.source.data.shape[0]
                 self.model.fs_signal = self.model.source.rate
-                self.model.tbin_signal = 1/self.model.fs_signal
+                self.model.tbin_signal = 1 / self.model.fs_signal
                 self.push5_0.setEnabled(False)
                 self.push6_0.setEnabled(False)
                 self.push7_0.setEnabled(False)
-            except:  #if not, opens warning dialog
+            except:  # if not, opens warning dialog
                 self.combo3.setCurrentIndex(self.combo3.findText('preprocessed'))
                 self.voltage_time_series()
                 NoHighGammaDialog()
-        self.model.updateCurXAxisPosition()    #updates time points
+        self.model.updateCurXAxisPosition()    # updates time points
         self.model.refreshScreen()
 
-
     def choose_stim(self):
         """Choose stimulus."""
         stimName = self.combo4.currentText()
         if stimName != '':
             self.model.refreshScreen()
 
-
-    ## Plot control buttons ----------------------------------------------------
+    # Plot control buttons ----------------------------------------------------
     def scroll_up(self):
         self.model.channel_Scroll_Up('unit')
 
@@ -848,13 +899,13 @@ class Application(QMainWindow):
         self.model.time_scroll(scroll=-1)
 
     def scroll_backward(self):
-        self.model.time_scroll(scroll=-1/3)
+        self.model.time_scroll(scroll=-1 / 3)
 
     def page_forward(self):
         self.model.time_scroll(scroll=1)
 
     def scroll_forward(self):
-        self.model.time_scroll(scroll=1/3)
+        self.model.time_scroll(scroll=1 / 3)
 
     def horizontalScaleIncrease(self):
         self.model.horizontalScaleIncrease()
@@ -943,18 +994,18 @@ class CustomViewBox(pg.ViewBox):
                 ev.ignore()
             else:
                 pg.ViewBox.mouseDragEvent(self, ev)
-                if ev.isStart():    #first click before dragging
+                if ev.isStart():    # first click before dragging
                     a = self.mapSceneToView(ev.scenePos())
-                    self.pos1 = a.x()              #initial x mark
-                    self.parent.model.DrawMarkTime(self.pos1)     #temporary line on x mark
-                elif ev.isFinish():  #release from dragging
-                    self.parent.model.RemoveMarkTime()   #remove line on x mark
+                    self.pos1 = a.x()              # initial x mark
+                    self.parent.model.DrawMarkTime(self.pos1)     # temporary line on x mark
+                elif ev.isFinish():  # release from dragging
+                    self.parent.model.RemoveMarkTime()   # remove line on x mark
                     a = self.mapSceneToView(ev.scenePos())
-                    self.pos2 = a.x()              #final x mark
+                    self.pos2 = a.x()              # final x mark
                     if self.pos1 is not None:
-                        if self.pos2 < self.pos1:     #marking from right to left
+                        if self.pos2 < self.pos1:     # marking from right to left
                             interval = [round(self.pos2, 3), round(self.pos1, 3)]
-                        else:               #marking from left to right
+                        else:               # marking from left to right
                             interval = [round(self.pos1, 3), round(self.pos2, 3)]
                         color = intervalsDict_[intervalType_]['color']
                         session = self.parent.current_session
@@ -963,15 +1014,54 @@ class CustomViewBox(pg.ViewBox):
                         self.parent.model.refreshScreen()
 
 
-def main(filename):  # If it was imported as a module
-    """Sets up QT application."""
+# If it is imported as a module
+def main(source_path='', metafile=None):
+    import sys
+
+    # Sets up QT application
     app = QCoreApplication.instance()
     if app is None:
         app = QApplication(sys.argv)  # instantiate a QtGui (holder for the app)
-    ex = Application(filename=filename)
+    ex = Application(source_path=source_path, metafile=metafile)
     sys.exit(app.exec_())
 
 
-if __name__ == '__main__':
-    main(sys.argv[0])
+def parse_arguments():
+    import argparse
 
+    parser = argparse.ArgumentParser(
+        description='Timeseries visualizer and data processing tools for Electrocorticography '
+                    '(ECoG) signals stored in NWB files, for Python.',
+    )
+
+    parser.add_argument(
+        "--source",
+        default='',
+        help="NWB file or HTK dir to be read."
+    )
+    parser.add_argument(
+        "--metafile",
+        default=None,
+        help="The path to the metadata YAML file."
+    )
+
+    # Parse arguments
+    args = parser.parse_args()
+
+    # File or dir to be loaded
+    source_path = args.source
+
+    # Metadata file (.yml)
+    metafile = args.metafile
+
+    return source_path, metafile
+
+
+def cmd_line_shortcut():
+    source_path, metafile = parse_arguments()
+    main(source_path=source_path, metafile=metafile)
+
+
+if __name__ == '__main__':
+    source_path, metafile = parse_arguments()
+    main(source_path=source_path, metafile=metafile)
